package com.trainpuzzle.controller;

import android.util.Log;

import com.trainpuzzle.exception.CannotPlaceTrackException;
import com.trainpuzzle.exception.CannotRemoveTrackException;
import com.trainpuzzle.exception.CannotRotateException;
import com.trainpuzzle.model.board.Board;
import com.trainpuzzle.model.board.Tile;
import com.trainpuzzle.model.board.Track;
import com.trainpuzzle.model.board.Landscape.LandscapeType;
import com.trainpuzzle.model.level.Level;

import com.trainpuzzle.model.level.Economy;
import com.trainpuzzle.model.board.TrackType;

public class TrackPlacer {

	private Level levelToAddTrack;
	private Economy economy;
	private Board map;
	
	public TrackPlacer(Level levelToAddTrack) {
		this.levelToAddTrack = levelToAddTrack;
		this.economy = levelToAddTrack.getEconomy();
		this.map = levelToAddTrack.getBoard();
	}
	
	public void placeTrack(Track track, int row, int column) throws CannotPlaceTrackException {
		Tile tile = map.getTile(row, column);
		int multiplier= tile.getLandscape().getMultiplier();
		System.out.println("landscape="+tile.getLandscape().getType());
		System.out.println("multiplier="+multiplier);
		String errorMessage = getPlaceTrackErrorMessage(tile, track);
		if(hasNoError(errorMessage)) {
			tile.setTrack(track);
			TrackType trackType = track.getTrackType();
			economy.useTracks(trackType, multiplier);
		}
		else {
			Log.w("CannotPlaceTrackException", errorMessage);
			throw new CannotPlaceTrackException(errorMessage);			
		}
	}
	
	
	private String getPlaceTrackErrorMessage(Tile tile, Track track) {
		String errorMessage = "";
		String commonMessage = "Failed to place track on a tile because ";
		
		if (tile.hasTrack()) {
			errorMessage = commonMessage + "there was already a track";
		} 
		else if (tile.hasObstacle()) {
			errorMessage = commonMessage + "there was an obstacle";
		} 
		else if (tile.getLandscapeType() == LandscapeType.WATER) {
			errorMessage = commonMessage + "there was a water landscape";
		}
		else if (!economy.isTrackAvailable(track.getTrackType(), 
				tile.getLandscape().getMultiplier())) {
			errorMessage = commonMessage + "the track is out of limit";
		}
		return errorMessage;
	}
	
	private boolean hasNoError(String errorMessage) {
		return errorMessage == "";
	}
	
	public void removeTrack(int row, int column) throws CannotRemoveTrackException {
		Tile tile = map.getTile(row, column);
		int multiplier= tile.getLandscape().getMultiplier();
		String errorMessage = getRemoveTrackErrorMessage(tile);
		if(hasNoError(errorMessage)) {
			TrackType trackType=tile.getTrack().getTrackType();
			tile.removeTrack();
			economy.returnTracks(trackType, multiplier);
		}
		else {
			Log.w("CannotRemoveTrackException", errorMessage);
			throw new CannotRemoveTrackException(errorMessage);
		}
	}
	
	private String getRemoveTrackErrorMessage(Tile tile) {
		String commonMessage = "Failed to be removed track from a tile because ";
		String errorMessage = "";
		if(!tile.hasTrack()) {
			errorMessage = commonMessage + "there was no track to be removed";
		}
		else if(!tile.getTrack().isRemovable()) {
			errorMessage = commonMessage + "it's an unremovable track";
		}
		return errorMessage;
	}
	
	public void rotateTrack(int row, int column) throws CannotRotateException {
		Tile tile = map.getTile(row, column);
		
		if(tile.hasTrack()) {
			tile.rotateTrack();
		}
		else {
			String errorMessage = "No track to be rotated on this tile";
			Log.w("CannotRotateTrackException", errorMessage);
			throw new CannotRotateException(errorMessage);
		}		
	}
	
	public Level getLevelWithTrack() {
		return levelToAddTrack;
	}
}